<?php


/**
 * Page callback for admin/dqx-adminmenu-data-json
 * Output administration menu for HTTP caching via AJAX request.
 */
function dqx_adminmenu_json() {
  if (!user_access('use dqx_adminmenu')) {
    $json = json_encode(array(
      'access' => FALSE,
      'alert' => "dqx_adminmenu access denied.",
    ));
  }
  else if ($GLOBALS['user']->uid != $_GET['uid']) {
    $uid_session = $GLOBALS['user']->uid;
    $uid_request = $_GET['uid'];
    $json = json_encode(array(
      'access' => FALSE,
      'alert' => "dqx_adminmenu access denied due to user id mismatch. Session says uid = $uid_session, request says uid = $uid_request.",
    ));
  }
  else {
    $cid = 'dqx_adminmenu:json:' . $GLOBALS['user']->uid . ':' . $GLOBALS['language']->language;
    $timestamp = variable_get('dqx_adminmenu_timestamp', 0);
    $cache = cache_get($cid)->data;
    if (!is_array($cache) || $cache['timestamp'] != $timestamp || TRUE) {
      $json = _dqx_adminmenu_json();
      $cache = array('timestamp' => $timestamp, 'json' => $json);
      cache_set($cid, $cache, 'cache', CACHE_TEMPORARY);
    }
    else {
      $json = $cache['json'];
    }
  }

  header("Content-type: application/json");
  header("Cache-Control: ''");  // HTTP/1.1
  $expires = date('D, d M Y H:i:s GMT', time() + 3600 * 24 * 2);  // two days ahead
  header("Expires: $expires");

  print $json;

  exit();
}


/**
 * Page callback for admin/dqx-adminmenu-data-json
 * Output administration menu for HTTP caching via AJAX request.
 */
function dqx_adminmenu_xml() {
  if (!user_access('use dqx_adminmenu')) {
    $xml = <<<EOT
      <div class="dqx_adminmenu-access_denied">dqx_adminmenu access denied</div>
EOT;
  }
  else if ($GLOBALS['user']->uid != $_GET['uid']) {
    $uid_session = $GLOBALS['user']->uid;
    $uid_request = $_GET['uid'];
    $xml = <<<EOT
      <div class="dqx_adminmenu-access_denied">dqx_adminmenu access denied due to user id mismatch. Session says uid = $uid_session, request says uid = $uid_request.</div>
EOT;
  }
  else {
    $cid = 'dqx_adminmenu:xml:' . $GLOBALS['user']->uid . ':' . $GLOBALS['language']->language;
    $cache = cache_get($cid)->data;
    
    // Timestamp of the latest menu rebuild.
    $timestamp = variable_get('dqx_adminmenu_timestamp', 0);

    if (!is_array($cache) || $cache['timestamp'] != $timestamp) {
      $xml = _dqx_adminmenu_html();
      $cache = array('timestamp' => $timestamp, 'xml' => $xml);
      cache_set($cid, $cache, 'cache', CACHE_TEMPORARY);
    }
    else {
      $xml = $cache['xml'];
    }
  }

  // header("Content-type: text/xml");
  header("Cache-Control: ''");  // HTTP/1.1
  $expires = date('D, d M Y H:i:s GMT', time() + 3600 * 24 * 2);  // two days ahead
  header("Expires: $expires");

  print $xml;

  exit();
}


function _dqx_adminmenu_json() {
  $tree_html = _dqx_adminmenu_tree_html();

  $html = !$tree_html ? '' : <<<EOT
    <div id="dqx_adminmenu">$tree_html</div>
EOT;

  $json = json_encode(array('html' => trim($html)));

  return $json;
}


function _dqx_adminmenu_html() {
  $tree_html = _dqx_adminmenu_tree_html();

  $html = !$tree_html ? '' : <<<EOT
    <div id="dqx_adminmenu">$tree_html</div>
EOT;

  return trim($html);
}


function _dqx_adminmenu_tree_html() {

  _dqx_adminmenu_init_plugin_dir();

  // build items and check access
  $items = module_invoke_all('dqx_adminmenu_items');

  // alter items and parent map
  $parent_map = array();
  $inline_children = array();
  $items_alter = new _dqx_adminmenu_ItemsAlter($items, $parent_map, $inline_children);
  module_invoke_all('dqx_adminmenu_items_alter', $items_alter);

  // build and alter submenus
  $submenus = dqx_adminmenu_build_structure($items, $parent_map);
  $structure_alter = new _dqx_adminmenu_StructureAlter($items, $submenus, $inline_children);

  module_invoke_all('dqx_adminmenu_structure_alter', $structure_alter);

  // display stuff.
  module_load_include('display.inc', 'dqx_adminmenu');

  $treeObject = new _dqx_adminmenu_MenuTree($submenus, $items, $inline_children);

  $ul_tree = $treeObject->renderMenu('admin');
  $ul_user = $treeObject->renderMenu('dqx-adminmenu-user');

  if ($ul_tree) {
    return $ul_user . $ul_tree;
  }
}


function _dqx_adminmenu_init_plugin_dir() {
  $plugin_dir = drupal_get_path('module', 'dqx_adminmenu') . '/plugins';
  foreach (scandir($plugin_dir) as $candidate) {
    if ($candidate != '.' && $candidate != '..' && is_file($plugin_dir . '/' . $candidate)) {
      if (preg_match('#^dqx_adminmenu\.([^\.]+)\.inc$#', $candidate, $m)) {
        if (module_exists($m[1])) {
          require_once $plugin_dir . '/' . $candidate;
        }
      }
    }
  }
}


function dqx_adminmenu_build_structure(array $items, array $parent_map) {
  // Normally, we determine the parent of an item by chopping off the last path fragment.
  // Some items should have a parent different from that.

  $paths = array_keys($items);
  $prefix_map = $parent_map + array_combine($paths, $paths);

  $submenus = array();
  foreach ($items as $path => $item) {
    if ($item['needs_approval']) {
      if (!_dqx_adminmenu_find_path_prefix($path, $parent_map)) {
        continue;
      }
    }
    $prefix = _dqx_adminmenu_find_path_prefix($path, $prefix_map);
    if (strlen($prefix)) {
      $submenus[$prefix][$path] = $path;
    }
  }

  return $submenus;
}


function _dqx_adminmenu_find_path_prefix($path, array $haystack) {
  $fragments = explode('/', $path);
  while (count($fragments) > 1) {
    array_pop($fragments);
    $prefix = implode('/', $fragments);
    if (isset($haystack[$prefix])) {
      return $haystack[$prefix];
    }
  }
}


function _dqx_adminmenu_build_menu_item($item) {

  if ($item['type'] == MENU_CALLBACK) {
    $item['hidden'] = -1;
  }
  elseif ($item['type'] == MENU_SUGGESTED_ITEM) {
    $item['hidden'] = 1;
  }

  // TODO: Why do we need this?
  $item['module'] = 'dqx_adminmenu';

  $item += array(
    'menu_name' => 'dqx_adminmenu',
    'link_title' => $item['title'],
    'link_path' => $item['path'],
    'hidden' => 0,
    'options' => empty($item['description']) ? array() : array(
      'attributes' => array('title' => $item['description'])
    ),

    'router_path' => $item['path'],

    // this is taken from menu_item_save().
    'weight' => 0,
    'has_children' => 0,
    'expanded' => 1,
    'customized' => 0,
    'updated' => 0,
  );

  // _menu_link_translate() wants serialized options
  $item['options'] = serialize($item['options']);

  // Check access
  // This does expect serialized options, but it does unserialize them again after.
  _menu_link_translate($item);

  return $item;
}


function _dqx_adminmenu_router_items($sql_filter, array $sql_args = array(), array $allow_wildcard = NULL) {

  $q = db_query("
    SELECT *
    FROM {menu_router}
    WHERE title != '' AND ($sql_filter)
    ORDER BY weight ASC, title ASC
  ", $sql_args);

  $allow_wildcard_regex = array();
  if (is_array($allow_wildcard)) {
    foreach ($allow_wildcard as $wildcard => $replace) {
      $regex = '#^' . preg_quote($wildcard) . '([^\%]*)$#';
      $allow_wildcard_regex[$regex] = $replace;
    }
  }

  $items = array();
  $callback_items = array();
  while ($item = db_fetch_array($q)) {
    if (strpos($item['path'], '%')) {
      if (is_array($allow_wildcard)) {
        $discard = TRUE;
        foreach ($allow_wildcard_regex as $regex => $replace) {
          if (preg_match($regex, $item['path'], $m)) {
            $item['path'] = $replace . $m[1];
            $discard = FALSE;
          }
        }
        if ($discard) {
          continue;
        }
      }
      else {
        continue;
      }
    }
    if ($item['type'] & (MENU_VISIBLE_IN_TREE | MENU_IS_LOCAL_TASK)) {
      $item = _dqx_adminmenu_build_menu_item($item);
      $items[$item['path']] = $item;
    }
    elseif ($item['type'] & MENU_VISIBLE_IN_BREADCRUMB && $item['number_parts'] > 2) {
      // $item['item_attributes']['style'] = 'background:green;';
      $item['needs_approval'] = TRUE;
      $item = _dqx_adminmenu_build_menu_item($item);
      $items[$item['path']] = $item;
    }
  }

  return $items;
}


abstract class _dqx_adminmenu_ItemsAlter_abstract {

  protected $_items;
  protected $_inline;

  function itemSetValue($k_item, $key, $value) {
    if (isset($this->_items[$k_item])) {
      $this->_items[$k_item][$key] = $value;
    }
  }

  function itemSetValues($k_item, array $values) {
    if (isset($this->_items[$k_item])) {
      foreach ($values as $k => $v) {
        $this->_items[$k_item][$k] = $v;
      }
    }
  }

  function itemSetNestedValue($k_item, $k1, $k2_or_value) {
    $args = func_get_args();
    $value = array_pop($args);
    $k_item = array_shift($args);
    if (isset($this->_items[$k_item])) {
      $this->_arraySetNestedValue($this->_items[$k_item], $args, $value);
    }
  }

  protected function _arraySetNestedValue(array &$arr, array $keys, $value) {
    if (!count($keys)) {
      // ouch
    }
    else {
      $key = array_shift($keys);
      if (!count($keys)) {
        $arr[$key] = $value;
      }
      else {
        if (!isset($arr[$key])) {
          $arr[$key] = array();
        }
        $this->_arraySetNestedValue($arr[$key], $keys, $value);
      }
    }
  }

  function itemSetWeight($k_item, $weight) {
    if (isset($this->_items[$k_item])) {
      $this->_items[$k_item]['weight'] = $weight;
    }
  }

  function itemSetTitle($k_item, $title) {
    if (isset($this->_items[$k_item])) {
      $this->_items[$k_item]['title'] = $title;
    }
  }

  function itemSetAttribute($k_item, $attr_key, $attr_value) {
    if (isset($this->_items[$k_item])) {
      $this->_items[$k_item]['item_attributes'][$attr_key] = $attr_value;
    }
  }

  function itemAddClass($k_item, $class) {
    if (isset($this->_items[$k_item])) {
      $this->_items[$k_item]['item_attributes']['class'] .= ' ' . $class;
    }
  }

  function getItem($k_item) {
    return $this->_items[$k_item];
  }

  function submenuSetInlineChild($k_parent, $k_child) {
    $this->_inline[$k_parent] = $k_child;
  }
}


class _dqx_adminmenu_ItemsAlter extends _dqx_adminmenu_ItemsAlter_abstract {

  protected $_parent_map;

  function __construct(&$items, &$parent_map, &$inline_children) {
    $this->_items = &$items;
    $this->_parent_map = &$parent_map;
    $this->_inline = &$inline_children;
  }

  function addItem($key, $item) {
    $this->_items[$key] = $item;
  }

  function removeItem($key) {
    unset($this->_items[$key]);
  }

  function parentMap($k_parent, $k_parent_replaced) {
    $this->_parent_map[$k_parent] = $k_parent_replaced;
  }
}


class _dqx_adminmenu_StructureAlter extends _dqx_adminmenu_ItemsAlter_abstract {

  protected $_submenus;

  function __construct(&$items, &$submenus, &$inline_children) {
    $this->_items = &$items;
    $this->_submenus = &$submenus;
    $this->_inline = &$inline_children;
  }

  function hasSubmenu($k_submenu) {
    return is_array($this->_submenus[$k_submenu]);
  }

  function setSubmenu($k_submenu, $submenu) {
    $this->_submenus[$k_submenu] = $submenu;
  }

  function submenuSetAttribute($k_submenu, $attr_name, $attr_value) {
    if (isset($this->_submenus[$k_submenu])) {
      $this->_items[$k_submenu]['submenu_attributes'][$attr_name] = $attr_value;
    }
  }

  function submenuAddClass($k_submenu, $class) {
    if (isset($this->_submenus[$k_submenu])) {
      $this->_items[$k_submenu]['submenu_attributes']['class'] .= ' ' . $class;
    }
  }

  function submenuGetPaths($k_submenu) {
    if (isset($this->_submenus[$k_submenu])) {
      return $this->_submenus[$k_submenu];
    }
    return array();
  }

  function submenuCountChildren($k_submenu) {
    if (isset($this->_submenus[$k_submenu])) {
      return count($this->_submenus[$k_submenu]);
    }
    return 0;
  }

  function submenuAddChild($k_submenu, $path) {
    if (isset($this->_submenus[$k_submenu]) || TRUE) {
      $this->_submenus[$k_submenu][$path] = $path;
    }
  }

  function submenuRemoveChild($k_submenu, $path) {
    unset($this->_submenus[$k_submenu][$path]);
  }
}





