
(function(){

  var d = Drupal.dqx_adminmenu;
  d.directional = {};


  d.squareSum = function(a, b, c) {
    var sum = 0;
    for (var i = 0; i < arguments.length; ++i) {
      sum += arguments[i] * arguments[i];
    }
    return sum;
  };


  d.euclidean = function(a, b, c) {
    var sum = 0;
    for (var i = 0; i < arguments.length; ++i) {
      sum += arguments[i] * arguments[i];
    }
    return Math.sqrt(sum);
  };


  d.directional.LazyDirectionTracker = function() {

    var prev = null;
    var px = 0.5;
    var py = 0.5;

    this.moveTo = function(x, y, t) {
      var event = {x:x, y:y, t:t};
      var delta = {x:0, y:0, t:0};
      if (prev) {
        delta.x = event.x - prev.x;
        delta.y = event.y - prev.y;
        delta.t = event.t - prev.t;
        if (delta.x > 0 && px < 1) {px = 1; --delta.x;}
        if (delta.x < 0 && px > 0) {px = 0; ++delta.x;}
        if (delta.y > 0 && py < 1) {py = 1; --delta.y;}
        if (delta.y < 0 && py > 0) {py = 0; ++delta.y;}
      }
      prev = event;
      return delta;
    };
  };


  d.directional.DirectionalityCourt = function() {

    var directionJudges = {};

    this.moveDelta = function(delta) {
      for (var k in directionJudges) {
        directionJudges[k].moveDelta(delta);
      }
    };

    this.hasDirectionUntil = function(k) {
      return directionJudges[k].hasDirectionUntil();
    };

    this.addDirection = function(k, T, f, debug_bg) {
      directionJudges[k] = new d.directional.DirectionalityJudge(T, f, debug_bg);
    };
  };


  d.directional.DirectionalityJudge = function(T, f, debug_bg) {

    if (debug_bg) {
      var $monitor = $('<li>').appendTo('#dqx_adminmenu-user');
      $monitor.css({background: debug_bg, 'border-left':'4px solid white', height:'10px'});
    }

    var dir = 0;

    this.moveDelta = function(delta) {
      var abs = {
        x: Math.abs(delta.x),
        y: Math.abs(delta.y),
        t: delta.t
      };

      // waiting
      dir -= delta.t / T;
      dir = Math.max(-0.2, dir);

      var dir_f = f(delta, abs);

      // movement
      dir += dir_f;
      dir = Math.max(-0.2, dir);
      dir = Math.min(1, dir);

      if (debug_bg) {
        $monitor.stop();
        $monitor.css('width', dir * 150 + 'px');
        $monitor.animate({width: 0}, dir * T);
      }
    };

    this.hasDirectionUntil = function() {
      return dir * T;
    };
  };


  d.MouseEngine_directional = function() {

    var tracker = new d.directional.LazyDirectionTracker();
    var court = new d.directional.DirectionalityCourt();

    court.addDirection('r', 450, function(delta, abs){return delta.x / 5 - abs.y / 9;});
    court.addDirection('l', 450, function(delta, abs){return - delta.x / 5 - abs.y / 9;});
    court.addDirection('ver', 100, function(delta, abs){return abs.y / 5 - abs.x / 8;});
    court.addDirection('up', 300, function(delta, abs){return - delta.y / 5 - abs.x / 8;});

    var timer;
    var $recentElement = true;
    var timeToChange;

    /**
     * @param $element
     *   The li, tr, #dqx_adminmenu or body element that is being hovered.
     * @param x, y
     *   x and y mouse position.
     */
    this.mouseMove = function($element, x, y) {
      var now = d.getMillis();
      var delta = tracker.moveTo(x, y, now);
      court.moveDelta(delta);
      var sameElement = d.compareElements($element, $recentElement);
      if (!sameElement || delta.x || delta.y) {
        var recent_depth = $recentElement ? d.countParentItems($recentElement) : 0;
        $recentElement = $element;
        var timeout = 300 + recent_depth * 160;
        if ($element && $element.is('.expandable')) {
          var depth = d.countParentItems($element);
          var lr = $element.parents('.dqx_adminmenu-left').length ? 'r' : 'l';
          switch (depth) {
            case 0:
              timeout = Math.max(court.hasDirectionUntil('ver'), court.hasDirectionUntil('up'));
              break;
            case 1:
            default:
              timeout = court.hasDirectionUntil(lr);
          }
        }
        var hard = (timeout > 0) ? false : true;
        var immediate = d.focusItem($element, hard);
        if (!immediate && (now + timeout < timeToChange || !sameElement)) {
          clearTimeout(timer);
          timer = setTimeout(function(){
            d.focusItem($recentElement, true);
          }, timeout);
        }
      }
    };
  };


  d.behaviors.mouse = function(context, settings, $divRoot) {
    var mouse_engine = new d.MouseEngine_directional();
    d.attachMouseEngine($divRoot, mouse_engine);
  };

})();
